@@ -6,16 +6,16 @@
 #' @import ggplot2
 #'
 #' @importFrom ggplot2 ggplot aes geom_polygon geom_line scale_colour_manual
-#' scale_fill_manual scale_x_date xlab ylab theme theme_grey element_text
+#' scale_fill_manual scale_x_date theme element_text
 #' element_blank element_rect element_line guides guide_legend coord_cartesian
-#' @importFrom grid unit
 #'
 #' @param data The function receives the full (list) output from the
 #' \code{\link{detect_event}} function.
 #' @param x This column is expected to contain a vector of dates as per the
-#' specification of \code{make_whole}. If a column headed \code{t} is present in
-#' the dataframe, this argument may be ommitted; otherwise, specify the name of
-#' the column with dates here.
+#' specification of \code{make_whole_fast}. If a column headed \code{t} is present in
+#' the dataframe, this argument may be omitted; otherwise, specify the name of
+#' the column with dates here. Note that this function will not work with
+#' hourly data.
 #' @param y This is a column containing the measurement variable. If the column
 #' name differs from the default (i.e. \code{temp}), specify the name here.
 #' @param min_duration The minimum duration (days) the event must be for it to
@@ -24,15 +24,9 @@
 #' (as per \code{metric}) detected within the time period specified by
 #' \code{start_date} and \code{end_date}. The default is 150 days.
 #' @param metric This tells the function how to choose the event that should be
-#' highlighted as the 'greatest' of the events in the chosen period. One may
-#' choose from the following options: \code{intensity_mean}, \code{intensity_max},
-#' \code{intensity_var},\code{intensity_cumulative}, \code{intensity_mean_relThresh},
-#' \code{intensity_max_relThresh}, \code{intensity_var_relThresh},
-#' \code{intensity_cumulative_relThresh}, \code{intensity_mean_abs},
-#' \code{intensity_max_abs}, \code{intensity_var_abs}, \code{intensity_cumulative_abs},
-#' \code{rate_onset}, \code{rate_decline}. Partial name matching is currently not
-#' supported so please specify the metric name precisely. The default is
-#' \code{intensity_cumulative}.
+#' highlighted as the 'greatest' of the events in the chosen period.
+#' Partial name matching is currently not supported so please specify the metric
+#' name precisely. The default is \code{intensity_cumulative}.
 #' @param start_date The start date of a period of time within which the largest
 #' event (as per \code{metric}) is retrieved and plotted. This may not necessarily
 #' correspond to the biggest event of the specified metric within the entire
@@ -54,6 +48,11 @@
 #' than "Temperature °C" (default)
 #' @param y_axis_range If one would like to control the y-axis range, provide the desired limits
 #' here as two numeric values (e.g. c(20, 30)).
+#' @param line_colours Provide a vector of colours here for the line geoms on the plot.
+#' The default for the base plot is c("black", "blue", "darkgreen"), and for categories
+#' it is: c("black", "gray20", "darkgreen", "darkgreen", "darkgreen", "darkgreen"). Note that
+#' three (\code{category} = FALSE) or six (\code{category} = TRUE) colours must be provided,
+#' with any colours in excess of the requirement being ignored.
 #'
 #' @return The function will return a line plot indicating the climatology,
 #' threshold and temperature, with the hot or cold events that meet the
@@ -76,27 +75,32 @@
 #' ts <- ts2clm(sst_WA, climatologyPeriod = c("1983-01-01", "2012-12-31"))
 #' res <- detect_event(ts)
 #'
-#' event_line(res, spread = 100, metric = "intensity_cumulative",
+#' event_line(res, spread = 100, metric = duration,
 #' start_date = "2010-12-01", end_date = "2011-06-30")
 #'
 #' event_line(res, spread = 100, start_date = "2010-12-01",
 #' end_date = "2011-06-30", category = TRUE)
 #'
+#' event_line(res, spread = 100, start_date = "2010-12-01",
+#' end_date = "2011-06-30", category = TRUE,
+#' line_colours = c("black", "blue", "gray20", "gray20", "gray20", "gray20"))
+#'
 event_line <- function(data,
                        x = t,
                        y = temp,
+                       metric = intensity_cumulative,
                        min_duration = 5,
                        spread = 150,
-                       metric = "intensity_cumulative",
                        start_date = NULL,
                        end_date = NULL,
                        category = FALSE,
                        x_axis_title = NULL,
                        x_axis_text_angle = NULL,
                        y_axis_title = NULL,
-                       y_axis_range = NULL) {
+                       y_axis_range = NULL,
+                       line_colours = NULL) {
 
-  date_end <- date_start <- duration <-  temp <-  NULL
+  date_end <- date_start <- duration <- temp <- intensity_cumulative <- NULL
 
   if (!(exists("event", data)) | !(exists("climatology", data)))
     stop("Please ensure you are running this function on the output of 'heatwaveR::detect_event()'")
@@ -106,56 +110,47 @@ event_line <- function(data,
   ts_y <- eval(substitute(y), data$climatology)
   data$climatology$ts_y <- ts_y
 
+  if (inherits(ts_x[1], "POSIXct"))
+    stop("event_line() will only work with daily data")
+
   if (is.null(start_date)) start_date <- min(data$climatology$ts_x)
   if (is.null(end_date)) end_date <- max(data$climatology$ts_x)
 
-  event <- data$event %>%
-    dplyr::filter(date_end >= start_date & date_start <= end_date) %>%
-    data.frame()
+  event <- data$event[data$event$date_end >= start_date & data$event$date_start <= end_date, ]
 
   if (nrow(event) == 0) stop("No events detected! Consider changing the 'start_date' or 'end_date' values.")
 
-  if (!(metric %in% c("intensity_mean", "intensity_max", "intensity_var", "intensity_cumulative", "intensity_mean_relThresh", "intensity_max_relThresh",
-                      "intensity_var_relThresh","intensity_cumulative_relThresh", "intensity_mean_abs", "intensity_max_abs", "intensity_var_abs",
-                      "intensity_cumulative_abs", "rate_onset", "rate_decline"))) {
-    stop("Please ensure you have spelled the desired metric correctly.")
-  }
-
   index_start <- index_end <- event_idx <-  NULL
 
-  event_idx <- as.vector(-abs(event[colnames(event) == metric])[,1])
-  event <- event[base::order(event_idx),]
-  event <- event %>%
-    dplyr::filter(duration >= min_duration) %>%
-    dplyr::mutate(index_start_fix = index_start - 1,
-                  index_end_fix = index_end + 1)
+  event_idx <- -abs(eval(substitute(metric), event))
+  event <- event[base::order(event_idx), ]
+  event <- event[event$duration >= min_duration, ]
+  event$index_start_fix <- event$index_start - 1
+  event$index_end_fix <- event$index_end + 1
 
   event_top <- event[1, ]
 
   date_spread <- seq((event_top$date_start - spread), (event_top$date_end + spread), by = "day")
 
-  event_sub <- event %>%
-    dplyr::filter(date_start >= min(date_spread),
-                  date_end <= max(date_spread))
+  event_sub <- event[event$date_start >= min(date_spread) & event$date_end <= max(date_spread), ]
 
   thresh_2x <- thresh_3x <- thresh_4x <- NULL
 
-  clim_diff <- data$climatology %>%
-    dplyr::mutate(diff = thresh - seas,
-           thresh_2x = thresh + diff,
-           thresh_3x = thresh_2x + diff,
-           thresh_4x = thresh_3x + diff)
+  clim_diff <- data$climatology
+  clim_diff$diff <- clim_diff$thresh - clim_diff$seas
+  clim_diff$thresh_2x <- clim_diff$thresh + clim_diff$diff
+  clim_diff$thresh_3x <- clim_diff$thresh_2x + clim_diff$diff
+  clim_diff$thresh_4x <- clim_diff$thresh_3x + clim_diff$diff
 
   clim_events <- data.frame()
   for (i in seq_len(nrow(event_sub))) {
-    clim_sub <- clim_diff[(event_sub$index_start_fix[i]):(event_sub$index_end_fix[i]),]
+    clim_sub <- clim_diff[(event_sub$index_start_fix[i]):(event_sub$index_end_fix[i]), ]
     clim_events <- rbind(clim_events, clim_sub)
   }
 
-  clim_top <- clim_diff[event_top$index_start_fix:event_top$index_end_fix,]
+  clim_top <- clim_diff[event_top$index_start_fix:event_top$index_end_fix, ]
 
-  clim_spread <- clim_diff %>%
-    dplyr::filter(ts_x %in% date_spread)
+  clim_spread <- clim_diff[clim_diff$ts_x %in% date_spread, ]
 
   thresh <- seas <- y1 <- y2 <-  NULL
 
@@ -174,7 +169,7 @@ event_line <- function(data,
   }
 
   if (!is.null(y_axis_title)) {
-    if(!is.character(y_axis_title)) stop("Please ensure that the argument provided to 'y_axis_title' is a character string.")
+    if (!is.character(y_axis_title)) stop("Please ensure that the argument provided to 'y_axis_title' is a character string.")
     ylabel <- y_axis_title
   } else {
     ylabel <- expression(paste("Temperature [", degree, "C]"))
@@ -188,7 +183,7 @@ event_line <- function(data,
   }
 
   if (!is.null(x_axis_text_angle)) {
-    if(!is.numeric(x_axis_text_angle)) stop("Please ensure that the argument provided to 'x_axis_text_angle' is a number.")
+    if (!is.numeric(x_axis_text_angle)) stop("Please ensure that the argument provided to 'x_axis_text_angle' is a number.")
     xtangle <- x_axis_text_angle
   } else {
     xtangle <- 0
@@ -199,20 +194,17 @@ event_line <- function(data,
     labs(x = xlabel, y = ylabel) +
     theme(plot.background = element_blank(),
           panel.background = element_rect(fill = "white"),
-          panel.border = element_rect(colour = "black", fill = NA, size = 0.75),
+          panel.border = element_rect(colour = "black", fill = NA, linewidth = 0.75),
           panel.grid.minor = element_line(colour = NA),
-          panel.grid.major = element_line(colour = "black", size = 0.2, linetype = "dotted"),
+          panel.grid.major = element_line(colour = "black", linewidth = 0.2, linetype = "dotted"),
           axis.text = element_text(colour = "black"),
-          axis.text.x = element_text(margin = unit(c(0.5, 0.5, 0.5, 0.5), "cm"),
-                                     angle = xtangle),
-          axis.text.y = element_text(margin = unit(c(0.5, 0.5, 0.5, 0.5), "cm")),
-          axis.ticks.length = unit(-0.25, "cm"),
+          axis.text.x = element_text(angle = xtangle),
           legend.background = element_rect(colour = "black"),
           legend.direction = "horizontal",
           legend.justification = c(0, 0),
-          legend.position = c(0.005, 0.015),
+          legend.position.inside = c(0.005, 0.015),
           legend.key = element_blank()
-          )
+    )
 
   if (category) {
 
@@ -225,6 +217,11 @@ event_line <- function(data,
       "4x Threshold" = "darkgreen"
     )
 
+    if (!is.null(line_colours)) {
+      if (!is.vector(line_colours)) stop("Please ensure that 'line_colours' is a vector (e.g. c('black', 'gray20', 'darkgreen', 'darkgreen', 'darkgreen', 'darkgreen')).")
+      lineColCat[seq_along(line_colours)] <- line_colours
+    }
+
     if (event_top$intensity_mean < 0) {
       fillColCat <- c(
         "Moderate" = "#C7ECF2",
@@ -233,13 +230,13 @@ event_line <- function(data,
         "Extreme" = "#111433"
       )
       ep <- ep +
-        geom_flame(data = clim_events, size = 0.5,
+        geom_flame(data = clim_events, linewidth = 0.5,
                    aes(x = ts_x, y = thresh, y2 = ts_y, fill = "Moderate")) +
-        geom_flame(data = clim_events, size = 0.5,
+        geom_flame(data = clim_events, linewidth = 0.5,
                    aes(x = ts_x, y = thresh_2x, y2 = ts_y, fill = "Strong")) +
-        geom_flame(data = clim_events, size = 0.5,
+        geom_flame(data = clim_events, linewidth = 0.5,
                    aes(x = ts_x, y = thresh_3x, y2 = ts_y, fill = "Severe")) +
-        geom_flame(data = clim_events, size = 0.5,
+        geom_flame(data = clim_events, linewidth = 0.5,
                    aes(x = ts_x, y = thresh_4x, y2 = ts_y, fill = "Extreme"))
     } else {
       fillColCat <- c(
@@ -249,35 +246,35 @@ event_line <- function(data,
         "Extreme" = "#2d0000"
       )
       ep <- ep +
-        geom_flame(data = clim_events, size = 0.5,
+        geom_flame(data = clim_events, linewidth = 0.5,
                    aes(x = ts_x, y = y1, y2 = y2, fill = "Moderate")) +
-        geom_flame(data = clim_events, size = 0.5,
+        geom_flame(data = clim_events, linewidth = 0.5,
                    aes(x = ts_x, y = y1, y2 = thresh_2x, fill = "Strong")) +
-        geom_flame(data = clim_events, size = 0.5,
+        geom_flame(data = clim_events, linewidth = 0.5,
                    aes(x = ts_x, y = y1, y2 = thresh_3x, fill = "Severe")) +
-        geom_flame(data = clim_events, size = 0.5,
+        geom_flame(data = clim_events, linewidth = 0.5,
                    aes(x = ts_x, y = y1, y2 = thresh_4x, fill = "Extreme"))
     }
 
     ep <- ep +
       geom_line(aes(y = thresh_2x, col = "2x Threshold"),
-                size = 0.7, linetype = "dashed") +
+                linewidth = 0.7, linetype = "dashed") +
       geom_line(aes(y = thresh_3x, col = "3x Threshold"),
-                size = 0.7, linetype = "dotdash") +
+                linewidth = 0.7, linetype = "dotdash") +
       geom_line(aes(y = thresh_4x, col = "4x Threshold"),
-                size = 0.7, linetype = "dotted") +
+                linewidth = 0.7, linetype = "dotted") +
       geom_line(aes(y = seas, col = "Climatology"),
-                size = 0.7, alpha = 1) +
+                linewidth = 0.7, alpha = 1) +
       geom_line(aes(y = thresh, col = "Threshold"),
-                size = 0.7, alpha = 1) +
-      geom_line(aes(y = ts_y, col = "Temperature"), size = 0.6) +
+                linewidth = 0.7, alpha = 1) +
+      geom_line(aes(y = ts_y, col = "Temperature"), linewidth = 0.6) +
       scale_colour_manual(name = NULL, values = lineColCat,
                           breaks = c("Temperature", "Climatology", "Threshold",
                                      "2x Threshold", "3x Threshold", "4x Threshold")) +
-      scale_fill_manual(name = NULL, values = fillColCat, guide = FALSE) +
+      scale_fill_manual(name = NULL, values = fillColCat, guide = "none") +
       guides(colour = guide_legend(override.aes = list(linetype = c("solid", "solid", "solid",
                                                                     "dashed", "dotdash", "dotted"),
-                                                       size = c(0.6, 0.7, 0.7, 0.7, 0.7, 0.7)))) +
+                                                       linewidth = c(0.6, 0.7, 0.7, 0.7, 0.7, 0.7)))) +
       theme(legend.direction = "vertical")
     ep
 
@@ -289,23 +286,28 @@ event_line <- function(data,
       "Threshold" = "darkgreen"
     )
 
+    if (!is.null(line_colours)) {
+      if (!is.vector(line_colours)) stop("Please ensure that 'line_colours' is a vector (e.g. c('black', 'blue', 'darkgreen')).")
+      lineCol[seq_along(line_colours)] <- line_colours
+    }
+
     ep <- ep +
-      geom_flame(data = clim_events, size = 0.5,
+      geom_flame(data = clim_events, linewidth = 0.5,
                  aes(x = ts_x, y = y1, y2 = y2, fill = "events")) +
-      geom_flame(data = clim_top, size = 0.5,
+      geom_flame(data = clim_top, linewidth = 0.5,
                  aes(x = ts_x, y = y1, y2 = y2, fill = "peak event")) +
       geom_line(aes(y = seas, col = "Climatology"),
-                size = 0.7, alpha = 1) +
+                linewidth = 0.7, alpha = 1) +
       geom_line(aes(y = thresh, col = "Threshold"),
-                size = 0.7, alpha = 1) +
-      geom_line(aes(y = ts_y, col = "Temperature"), size = 0.6) +
+                linewidth = 0.7, alpha = 1) +
+      geom_line(aes(y = ts_y, col = "Temperature"), linewidth = 0.6) +
       scale_colour_manual(name = NULL, values = lineCol,
                           breaks = c("Temperature", "Climatology", "Threshold")) +
-      scale_fill_manual(name = NULL, values = fillCol, guide = FALSE)
+      scale_fill_manual(name = NULL, values = fillCol, guide = "none")
 
-    if(!is.null(y_axis_range)){
-      if(length(y_axis_range)!=2) stop("Please ensure that exactly two numbers are provided to 'y_axis_range' (e.g. c(10, 20)).")
-      if(!is.numeric(y_axis_range[1]) | !is.numeric(y_axis_range[2]))
+    if (!is.null(y_axis_range)) {
+      if (length(y_axis_range)!=2) stop("Please ensure that exactly two numbers are provided to 'y_axis_range' (e.g. c(10, 20)).")
+      if (!is.numeric(y_axis_range[1]) | !is.numeric(y_axis_range[2]))
         stop("Please ensure that only numeric values are provided to 'y_axis_range'.")
       ep <- ep + coord_cartesian(ylim = c(y_axis_range[1], y_axis_range[2]))
     }
@@ -317,25 +319,28 @@ event_line <- function(data,
 #'
 #' Visualise a timeline of several possible event metrics as 'lollipop' graphs.
 #'
-#' @importFrom ggplot2 aes_string geom_segment geom_point scale_x_continuous
+#' @importFrom ggplot2 geom_segment geom_point scale_x_continuous
 #' element_rect element_line labs scale_y_continuous
-#' @importFrom grid unit
 #'
 #' @param data Output from the \code{\link{detect_event}} function.
-#' @param xaxis One of \code{event_no}, \code{date_start} or \code{date_peak}.
-#' Default is \code{date_start}.
-#' @param metric One of \code{intensity_mean}, \code{intensity_max},
+#' @param xaxis The name of a column from the \code{event} data.frame in
+#' the output of \code{\link{detect_event}}. Suggested choices are, but not
+#' limited to, of \code{event_no}, \code{date_start} or \code{date_peak}.
+#' Default is \code{date_peak}.
+#' @param metric The name of a column from the \code{event} data.frame in
+#' the output of \code{\link{detect_event}}.Suggested choices are, but not
+#' limited to, \code{intensity_mean}, \code{intensity_max},
 #' \code{intensity_cumulative} and \code{duration}.
 #'  Default is \code{intensity_max}.
 #' @param event_count The number of top events to highlight, as determined by the
-#' value given to \code{metric}. Default is 3.
+#' column given to \code{metric}. Default is 3.
 #'
 #' @return The function will return a graph of the intensity of the selected
 #' \code{metric} along the y-axis and the chosen \code{xaxis} value.
 #' The number of top events as per \code{event_count} will be highlighted
 #' in a brighter colour. This function differs in use from \code{\link{geom_lolli}}
 #' in that it creates a stand-alone figure. The benefit of this being
-#' that one must not have any prior knowledge of \code{ggplot2} to create the figure.
+#' that one does not need any prior knowledge of \code{ggplot2} to create the figure.
 #'
 #' @author Albertus J. Smit and Robert W. Schlegel
 #'
@@ -351,28 +356,29 @@ event_line <- function(data,
 #' lolli_plot(res)
 #'
 lolli_plot <- function(data,
-                       xaxis = "date_peak",
-                       metric = "intensity_max",
+                       xaxis = date_peak,
+                       metric = intensity_max,
                        event_count = 3) {
 
+  date_peak <- date_start <- duration <- intensity_max <-  NULL
+
   if (!(exists("event", data)) | !(exists("climatology", data))) stop("Please ensure you are running this function on the output of 'heatwaveR::detect_event()'")
 
-  if (!(metric %in% c("intensity_mean", "intensity_max", "intensity_cumulative", "duration"))) {
-    stop("Please ensure you have spelled the name of desired metric correctly.")
-  }
-
-  if (!(xaxis %in% c("event_no", "date_start", "date_peak"))) {
-    stop("Please ensure you have spelled the name of desired x-axis correctly.")
-  }
-
   if (event_count > nrow(data$event)) {
     stop("Please ensure that event_count is less or equal to than the total number of events in your results.")
   }
 
-  event <- data$event %>%
-    dplyr::select(metric, xaxis)
+  ts_x <- eval(substitute(xaxis), data$event)
+  data$event$ts_x <- ts_x
+  ts_y <- eval(substitute(metric), data$event)
+  data$event$ts_y <- ts_y
 
-  y_top <- as.numeric(event[which(abs(event[ ,1]) == max(abs(event[ ,1])))[1], 1]) * 1.05
+  event <- data$event
+
+  y_max <- max(as.numeric(event$ts_y), na.rm = T)
+  y_min <- min(as.numeric(event$ts_y), na.rm = T)
+  if (abs(y_min) > abs(y_max)) y_top <- y_min * 1.05
+  if (abs(y_max) > abs(y_min)) y_top <- y_max * 1.05
   if (y_top >= 0) y_limits <- c(0, y_top)
   if (y_top < 0) y_limits <- c(y_top, 0)
 
@@ -382,33 +388,16 @@ lolli_plot <- function(data,
     lolli_col <- c("salmon", "red")
   }
 
-  if (xaxis == "event_no") xlabel <- "Event number"
-  if (xaxis == "date_start") xlabel <- "Start date"
-  if (xaxis == "date_peak") xlabel <- "Peak date"
-
-  if (metric == "intensity_max") ylabel <- expression(paste("Maximum intensity [", degree, "C]"))
-  if (metric == "intensity_mean") ylabel <- expression(paste("Mean intensity [", degree, "C]"))
-  if (metric == "intensity_cumulative") ylabel <- expression(paste("Cumulative intensity [", degree, "C x days]"))
-  if (metric == "duration") ylabel <- "Duration [days]"
-
-  lolli <- ggplot(data = event, aes_string(x = xaxis, y = metric)) +
+  lolli <- ggplot(data = event, aes(x = ts_x, y = ts_y)) +
     geom_lolli(colour = lolli_col[1], colour_n = lolli_col[2], fill = "grey70", n = event_count) +
-    labs(x = xlabel, y = ylabel) +
+    labs(x = enquo(xaxis), y = enquo(metric)) +
     scale_y_continuous(expand = c(0, 0), limits = y_limits) +
     theme(plot.background = element_blank(),
           panel.background = element_rect(fill = "white"),
-          panel.border = element_rect(colour = "black", fill = NA, size = 0.75),
+          panel.border = element_rect(colour = "black", fill = NA, linewidth = 0.75),
           panel.grid.minor = element_line(colour = NA),
-          panel.grid.major = element_line(colour = "black", size = 0.2, linetype = "dotted"),
-          axis.text = element_text(colour = "black"),
-          axis.text.x = element_text(margin = unit(c(0.5, 0.5, 0.5, 0.5), "cm")),
-          axis.text.y = element_text(margin = unit(c(0.5, 0.5, 0.5, 0.5), "cm")),
-          axis.ticks.length = unit(-0.25, "cm")
+          panel.grid.major = element_line(colour = "black", linewidth = 0.2, linetype = "dotted"),
+          axis.text = element_text(colour = "black")
           )
-
-  if (xaxis == "event_no") {
-    lolli <- lolli +
-      scale_x_continuous(breaks = seq(from = 0, to = nrow(data$event), by = 5))
-  }
   lolli
 }
